@@ -4,7 +4,9 @@ require 5.00503;
 use strict;
 # use warnings; # commented out for 5.005 compatibility
 use Carp;
+use Socket;
 use Data::Dumper;
+use Sys::Hostname;
 use Net::Peep::Log;
 use Net::Peep::Host;
 
@@ -16,7 +18,7 @@ use vars qw{ @ISA %EXPORT_TAGS @EXPORT_OK @EXPORT $VERSION };
 %EXPORT_TAGS = ( 'all' => [ qw( ) ] );
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 @EXPORT = qw( );
-$VERSION = do { my @r = (q$Revision$ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 1.8 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 sub new {
 
@@ -24,19 +26,26 @@ sub new {
 	my $class = ref($self) || $self;
 	my $this = {};
 	bless $this, $class;
-	return $this;
 
 } # end sub new
 
-# returns a logging object
 sub logger {
 
-	my $self = shift;
-	if ( ! exists $self->{'__LOGGER'} ) { $self->{'__LOGGER'} = new Net::Peep::Log }
-	return $self->{'__LOGGER'};
+    # returns a logging object
+    my $self = shift;
+    if ( ! exists $self->{'__LOGGER'} ) { $self->{'__LOGGER'} = new Net::Peep::Log }
+    return $self->{'__LOGGER'};
 
 } # end sub logger
 
+sub client {
+
+    my $self = shift;
+    if (@_) { $self->{'CLIENT'} = shift; }
+    return $self->{'CLIENT'};
+
+} # end sub client
+
 sub setVersion {
 
 	my $self = shift;
@@ -66,6 +75,35 @@ sub versionExists {
 
 } # end sub versionExists
 
+sub setSoundPath {
+
+	my $self = shift;
+	my $soundpath = shift || confess "Cannot set sound path:  No sound path information found";
+	$self->{"__SOUNDPATH"} = $soundpath;
+	$self->logger()->debug(1,"Configuration file soundpath [$soundpath] identified.");
+	return 1;
+
+} # end sub setSoundPath
+
+sub getSoundPath {
+
+	my $self = shift;
+
+	if (exists $self->{"__SOUNDPATH"}) {
+		return $self->{"__SOUNDPATH"};
+	} else {
+		confess "Cannot get sound path:  No sound path information has been set.";
+	}
+
+} # end sub getSoundPath
+
+sub soundPathExists {
+
+	my $self = shift;
+	return exists $self->{"__SOUNDPATH"};
+
+} # end sub soundPathExists
+
 sub setApp {
 
 	my $self = shift;
@@ -105,8 +143,8 @@ sub getClientPort {
 
 	if (exists $self->{"__PORT"}->{$client}) {
 		return $self->{"__PORT"}->{$client};
-	} elsif ($self->optionExists('port')) {
-		return $self->getOption('port');
+	} elsif ($self->optionExists($client,'port')) {
+		return $self->getOption($client,'port');
 	} else {
 		confess "Cannot get port:  No port information has been defined for the client [$client].";
 	}
@@ -336,7 +374,7 @@ sub setConfigurationText {
 
 	$self->{"__CONFIGURATIONTEXT"}->{$client} = join '', @text;
 
-	$self->logger()->debug(1,"Configuration text of " .
+	$self->logger()->debug(1,"\tConfiguration text of length " .
 		length($self->{"__CONFIGURATIONTEXT"}->{$client}) .
 		" added to client [$client].");
 
@@ -356,6 +394,40 @@ sub getConfigurationText {
 
 } # end sub getConfigurationText
 
+sub setNotificationText {
+
+	my $self = shift;
+	my $client = shift;
+	my @text = @_;
+
+	confess "Cannot set notification text:  No client found."
+		unless $client;
+
+	confess "Cannot set notification text:  No text found."
+		unless @text;
+
+	$self->{"__NOTIFICATIONTEXT"}->{$client} = join '', @text;
+
+	$self->logger()->debug(1,"Notification text of length " .
+		length($self->{"__NOTIFICATIONTEXT"}->{$client}) .
+		" added to client [$client].");
+
+	return 1;
+
+} # end sub setNotificationText
+
+sub getNotificationText {
+
+	my $self = shift;
+	my $client = shift;
+
+	confess "Cannot get notification text:  It has not been set yet."
+		unless exists $self->{"__NOTIFICATIONTEXT"}->{$client};
+
+	return $self->{"__NOTIFICATIONTEXT"}->{$client};
+
+} # end sub getNotificationText
+
 sub addState {
 
 	my $self = shift;
@@ -454,6 +526,7 @@ sub getClientEvents {
 sub checkClientEvent {
 
 	my $self = shift;
+	my $client = shift || confess "Client not found";
 	my $event = shift || confess "Event not found";
 
 	my $return = 0;
@@ -467,9 +540,9 @@ sub checkClientEvent {
 	my @exclude = ();
 	my @events = ();
 
-	@events = split //, $self->getOption('events') if $self->optionExists('events');
-	@groups = @{ $self->getOption('groups') } if $self->optionExists('groups');
-	@exclude = @{ $self->getOption('exclude') } if $self->optionExists('exclude');
+	@events = split //, $self->getOption($client,'events') if $self->optionExists($client,'events');
+	@groups = @{ $self->getOption($client,'groups') } if $self->optionExists($client,'groups');
+	@exclude = @{ $self->getOption($client,'exclude') } if $self->optionExists($client,'exclude');
 
 	# first check the events option
 	
@@ -492,22 +565,71 @@ sub checkClientEvent {
 
 } # end sub checkClientEvent
 
+sub checkClientHost {
+
+	my $self = shift;
+	my $client = shift || confess "Client not found";
+	my $host = shift || confess "Host not found";
+
+	my $return = 0;
+
+	my $event = $host->getEvent();
+
+	return $self->checkClientEvent($client,$event);
+
+} # end sub checkClientHost
+
 sub addClientHost {
 
 	my $self = shift;
-	my $name = shift || confess "Cannot add client host:  No client host identifier found";
+	my $client = shift || confess "Cannot add client host:  No client identifier found";
 	my $value = shift || confess "Cannot add client host:  No client host information found";
 
-	confess "Cannot set client host [$name]:  Expecting a hash ref (instead of [$value])."
+	confess "Cannot set client host for client [$client]:  Expecting a hash ref (instead of [$value])."
 		unless ref($value) eq 'HASH';
 
-	my $ip = gethostbyname($value->{'name'});
+	my $identifier = $value->{'host'};
+
+	my ($iaddr,$host,$ip);
+	if ($identifier =~ /^(\d+\.)+\d+$/) {
+	    # we were given an IP address
+	    $ip = $identifier;
+	    $host = inet_aton($ip);
+	    $host = gethostbyaddr($host,AF_INET) if $host;
+	    $self->logger()->log("\t\tThe host name for IP [$ip] could not be found.  This host will be ignored.")
+		and return 0 unless $host;
+	    $self->logger()->debug(9,"\t\tThe host name [$host] was found for host [$identifier].");
+	} elsif ($identifier =~ /^([\w-]+\.)+\w+$/) {
+	    # we were given a host name
+	    $host = $identifier;
+	    $ip = gethostbyname($identifier);
+	    # funny that the next line and previous line can't be combined ... but Socket complains!
+	    $ip = inet_ntoa($ip) if $ip;
+	    $self->logger()->log("\t\tThe IP address for host [$identifier] could not be found.  This host will be ignored.")
+		and return 0 unless $ip;
+	    $self->logger()->debug(9,"\t\tThe IP address [$ip] was found for host [$identifier].");
+	} else {
+	    $self->logger()->log("The host name or IP [$identifier] does not appear to be valid.  This host will be ignored.");
+	    return;
+	}
+	    
+	my $event = {
+	    name => $value->{'name'},
+	    group => $value->{'group'},
+	    'option-letter' => $value->{'option-letter'},
+	    location => $value->{'location'},
+	    priority => $value->{'priority'},
+	    status => $value->{'status'},
+
+	};
 
 	my $clienthost = new Net::Peep::Host;
-	$clienthost->setName($value->{'name'});
-	$clienthost->setEvent($value->{'event'});
+	$clienthost->setName($host);
+	$clienthost->setIP($ip);
+	$clienthost->setEvent($event);
+	$clienthost->setNotificationLevel($value->{'status'});
 
-	push @ { $self->{"__CLIENTHOST"}->{$name} }, $value;
+	push @ { $self->{"__CLIENTHOST"}->{$client} }, $clienthost;
 
 	return 1;
 
@@ -517,7 +639,7 @@ sub getClientHostList {
 
 	my $self = shift;
 
-	confess "Cannot get clienthost list:  No clienthost information has been set." 
+	confess "Cannot get clienthost list:  No client host information has been set." 
 		unless exists $self->{"__CLIENTHOST"};
 
 	my @clienthosts;
@@ -533,54 +655,210 @@ sub getClientHostList {
 sub getClientHosts {
 
 	my $self = shift;
-	my $name = shift || confess "Cannot get clienthost:  No clienthost identifier found";
+	my $client = shift || confess "Cannot get client host:  No client identifier found";
 
-	confess "Cannot get information for the clienthost [$name]:  No information has been set."
-		unless exists $self->{"__CLIENTHOST"} && exists $self->{"__CLIENTHOST"}->{$name};
+	$self->logger()->log("Cannot get host information for the client [$client]:  No information has been set.")
+	    and return
+		unless exists $self->{"__CLIENTHOST"} && exists $self->{"__CLIENTHOST"}->{$client};
 
-	return $self->{"__CLIENTHOST"}->{$name};
+	return wantarray ? @{$self->{"__CLIENTHOST"}->{$client}} : $self->{"__CLIENTHOST"}->{$client};
 
 } # end sub getClientHosts
 
-sub optionExists {
+sub addClientUptime {
 
 	my $self = shift;
-	my $option = shift || confess "Cannot determine existence of option:  Option name not specified.";
-	return exists $self->{"__OPTIONS"}->{$option};
+	my $client = shift || confess "Cannot add client uptime:  No client identifier found";
+	my $value = shift || confess "Cannot add client uptime:  No client uptime information found";
 
-} # end sub optionExists
+	confess "Cannot set client uptime setting for client [$client]:  Expecting a hash ref (instead of [$value])."
+		unless ref($value) eq 'HASH';
+
+	confess "Cannot set client uptime setting for client [$client]:  The hash ref is missing important keys."
+	    unless exists($value->{'name'}) && exists($value->{'value'}) && exists($value->{'status'});
+
+	push @ { $self->{"__CLIENTUPTIME"}->{$client} }, $value;
+
+	return 1;
+
+} # end sub addClientUptime
+
+sub getClientUptimeList {
+
+	my $self = shift;
+
+	confess "Cannot get client uptime settings list:  No client uptime information has been set." 
+		unless exists $self->{"__CLIENTUPTIME"};
+
+	my @clientuptimes;
+
+	for my $client (keys % { $self->{"__CLIENTUPTIME"} }) {
+		push @clientuptimes, @ { $self->{"__CLIENTUPTIME"}->{$client} };
+	}
+
+	return wantarray ? @clientuptimes : [@clientuptimes];
+
+} # end sub getClientUptimeList
+
+sub getClientUptimes {
+
+	my $self = shift;
+	my $client = shift || confess "Cannot get client uptime settings:  No client identifier found";
+
+	$self->logger()->log("Cannot get uptime information for the client [$client]:  No information has been set.")
+	    and return
+		unless exists $self->{"__CLIENTUPTIME"} && exists $self->{"__CLIENTUPTIME"}->{$client};
+
+	return wantarray ? @{$self->{"__CLIENTUPTIME"}->{$client}} : $self->{"__CLIENTUPTIME"}->{$client};
+
+} # end sub getClientUptimes
+
+sub addClientProc {
+
+	my $self = shift;
+	my $client = shift || confess "Cannot add client proc:  No client identifier found";
+	my $value = shift || confess "Cannot add client proc:  No client proc information found";
+
+	confess "Cannot set client proc setting for client [$client]:  Expecting a hash ref (instead of [$value])."
+		unless ref($value) eq 'HASH';
+
+	confess "Cannot set client proc setting for client [$client]:  The hash ref is missing important keys."
+	    unless exists($value->{'name'}) && exists($value->{'value'}) && exists($value->{'status'});
+
+	push @ { $self->{"__CLIENTPROC"}->{$client} }, $value;
+
+	return 1;
+
+} # end sub addClientProc
+
+sub getClientProcList {
+
+	my $self = shift;
+
+	confess "Cannot get client proc settings list:  No client proc information has been set." 
+		unless exists $self->{"__CLIENTPROC"};
+
+	my @clientprocs;
+
+	for my $client (keys % { $self->{"__CLIENTPROC"} }) {
+		push @clientprocs, @ { $self->{"__CLIENTPROC"}->{$client} };
+	}
+
+	return wantarray ? @clientprocs : [@clientprocs];
+
+} # end sub getClientProcList
+
+sub getClientProcs {
+
+	my $self = shift;
+	my $client = shift || confess "Cannot get client proc settings:  No client identifier found";
+
+	$self->logger()->log("Cannot get proc information for the client [$client]:  No information has been set.")
+	    and return
+		unless exists $self->{"__CLIENTPROC"} && exists $self->{"__CLIENTPROC"}->{$client};
+
+	return wantarray ? @{$self->{"__CLIENTPROC"}->{$client}} : $self->{"__CLIENTPROC"}->{$client};
+
+} # end sub getClientProcs
 
 sub setOption {
 
-	my $self = shift;
-	my $option = shift || confess "Cannot set option:  Option name not specified.";
-	if (@_) {
-		my $value = shift;
-		$self->{"__OPTIONS"}->{$option} = $value;
-		return 1;
-	} else {
-		confess "Cannot set option [$option]:  The option value was not found.";
-	}
+    my $self = shift;
+
+    # The following bit of logic is a bit of a kludge.  If you're
+    # wondering why it was done, please contact the author :-)
+
+    if (@_ == 2) {
+
+	my $client = $self->client() || confess "Cannot set client option:  Client not specified.";
+	my $name = $client->name() || confess "Cannot set client option:  Client name not specified.";
+	my $option = shift || confess "Cannot set client option:  Option name not specified.";
+	my $value = shift;
+	$self->{"__OPTIONS"}->{$name}->{$option} = $value;
+	return 1;
+
+    } elsif (@_ == 3) {
+
+	my $client = shift || confess "Cannot set client option:  Client name not specified.";
+	my $option = shift || confess "Cannot set client option:  Option name not specified.";
+	my $value = shift;
+	$self->{"__OPTIONS"}->{$client}->{$option} = $value;
+	return 1;
+
+    } else {
+
+	confess "Cannot set client option:  Wrong number of arguments to setOption method.";
+
+    }
 
 } # end sub setOption
 
 sub getOption {
 
 	my $self = shift;
-	my $option = shift || confess "Cannot get option:  Option name not specified.";
+	my $name;
+	my $option;
+	if (@_ == 1) {
+	    $option = shift || confess "Cannot get client option:  Option name not specified.";
+	    my $client = $self->client() || confess "Cannot get client option:  Client not specified.";;
+	    $name = $client->name() || confess "Cannot get client option:  Client name not specified.";
+	} elsif (@_ == 2) {
+	    $name = shift || confess "Cannot get client option:  Client name not specified.";
+	    $option = shift || confess "Cannot get client option:  Option name not specified.";
+	} else {
+	    confess "Cannot get option [$option]:  Incorrect number of arguments to the getOption method.";
+	}
+#	$self->logger()->debug(9,"Getting option [$option] for client [$name] ...");
 	confess "Cannot get option [$option]:  The option value has not been set yet."
-		unless exists $self->{"__OPTIONS"}->{$option};
-	return $self->{"__OPTIONS"}->{$option};
+	    unless $self->optionExists($name,$option);
+	return $self->{"__OPTIONS"}->{$name}->{$option};
 
-} # end sub option
+} # end sub getOption
+
+sub optionExists {
+
+    # The following bit of logic is a bit of a kludge.  If you're
+    # wondering why it was done, please contact the author :-)
+
+    my $self = shift;
+
+    my $option;
+    my $name;
+    if (@_ == 1) {
+	$option = shift || confess "Cannot evaluate client option:  Option name not specified.";
+	my $client = $self->client() || confess "Cannot evaluate client option:  Client not specified.";;
+	$name = $client->name() || confess "Cannot evaluate client option:  Client name not specified.";
+    } elsif (@_ == 2) {
+	$name = shift || confess "Cannot evaluate client option:  Client name not specified.";
+	$option = shift || confess "Cannot evaluate client option:  Option name not specified.";
+    } else {
+	confess "Cannot evaluate client option:  Wrong number of arguments to optionExists method.";
+    }
+#    $self->logger()->debug(9,"Checking existence of option [$option] for client [$name] ....");
+    if (exists $self->{"__OPTIONS"}->{$name}
+	and exists $self->{"__OPTIONS"}->{$name}->{$option}) {
+	return 1;
+    } else {
+	return 0;
+    }
+
+} # end sub optionExists
 
 sub getOptions {
 
 	# returns the names of all of the currently set options
 	my $self = shift;
+	my $name;
+	if (@_) {
+	    $name = shift;
+	} else {
+	    my $client = $self->client() || confess "Cannot get client options:  Client not specified.";;
+	    $name = $client->name() || confess "Cannot get client options:  Client name not specified.";
+	}
+	return () unless exists $self->{"__OPTIONS"}->{$name};
 	return wantarray 
-		? ( keys % { $self->{"__OPTIONS"} } )
-		: [ keys % { $self->{"__OPTIONS"} } ];
+		? ( keys % { $self->{"__OPTIONS"}->{$name} } )
+		: [ keys % { $self->{"__OPTIONS"}->{$name} } ];
 
 } # end sub getOptions
 
@@ -589,10 +867,17 @@ sub getOptionsHash {
 	# returns the names of all of the currently set options
 	my $self = shift;
 	my %return;
-	for my $option (keys % { $self->{"__OPTIONS"} }) {
-		$return{$option} = $self->getOption($option);
+	my $name;
+	if (@_) {
+	    $name = shift;
+	} else {
+	    my $client = $self->client() || confess "Cannot get options hash:  Client not specified.";;
+	    $name = $client->name() || confess "Cannot get options hash:  Client name not specified.";
 	}
-	return % { $self->{"__OPTIONS"} };
+	for my $option (keys % { $self->{"__OPTIONS"}->{$name} }) {
+		$return{$option} = $self->getOption($name,$option);
+	}
+	return %return;
 
 } # end sub getOptionHash
 
@@ -671,9 +956,19 @@ None by default.
   setState($event,$hashref) - Associates the state $state with the
   information contained in $hashref.
 
-  setConfigurationText($text) - Sets the configuration text to $text.
+  setConfigurationText($client,$text) - Sets the configuration text to
+  $text for client $client.  For reference and debugging only.  The
+  text is not actually used internally for any purpose.
 
-  getConfigurationText() - Returns the configuration text.
+  getNotificationText($client) - Returns the configuration text for
+  client $client.
+
+  setNotificationText($client,$text) - Sets the configuration text to
+  $text for client $client.  For reference and debugging only.  The
+  text is not actually used internally for any purpose.
+
+  getConfigurationText($client) - Returns the configuration text for
+  client $client.
 
   setClientEvent($name,$arrayref) - Sets the events associated with
   the client $name.
@@ -692,11 +987,20 @@ None by default.
 
   getVersion() - Return the version number taken from the Peep configuration
   file (e.g., peep.conf) if it exists.  Confesses if the version has not been
-  set yet.
+  set yet.  See also versionExists().
 
   versionExists() - Returns 1 if a version has been set with the setVersion()
   method, 0 otherwise.
 
+  setSoundPath() - Sets the path to the Peep sound respository
+  (typically /usr/local/share/peep/sounds).
+
+  getSoundPath() - Returns the sound path.  Confesses if the sound
+  path has not been set yet.  See also soundPathExists().
+
+  soundPathExists() - Returns 1 if the sound path has been set with
+  the setSoundPath() method, 0 otherwise.
+
 =head1 AUTHOR
 
 Collin Starkweather Copyright (C) 2001
@@ -710,6 +1014,22 @@ http://peep.sourceforge.net
 =head1 CHANGE LOG
 
 $Log: Conf.pm,v $
+Revision 1.8  2001/09/23 08:53:56  starky
+The initial checkin of the 0.4.4 release candidate 1 clients.  The release
+includes (but is not limited to):
+o A new client:  pinger
+o A greatly expanded sysmonitor client
+o An API for creating custom clients
+o Extensive documentation on creating custom clients
+o Improved configuration file format
+o E-mail notifications
+Contact Collin at collin.starkweather@colorado with any questions.
+
+Revision 1.7  2001/08/08 20:17:57  starky
+Check in of code for the 0.4.3 client release.  Includes modifications
+to allow for backwards-compatibility to Perl 5.00503 and a critical
+bug fix to the 0.4.2 version of Net::Peep::Conf.
+
 Revision 1.6  2001/08/06 04:20:35  starky
 Fixed bug #447844.
 
@@ -775,6 +1095,22 @@ a work-out-of-the-box system.
 Revision 1.6  =head1 CHANGE LOG
  
 $Log: Conf.pm,v $
+Revision 1.8  2001/09/23 08:53:56  starky
+The initial checkin of the 0.4.4 release candidate 1 clients.  The release
+includes (but is not limited to):
+o A new client:  pinger
+o A greatly expanded sysmonitor client
+o An API for creating custom clients
+o Extensive documentation on creating custom clients
+o Improved configuration file format
+o E-mail notifications
+Contact Collin at collin.starkweather@colorado with any questions.
+
+Revision 1.7  2001/08/08 20:17:57  starky
+Check in of code for the 0.4.3 client release.  Includes modifications
+to allow for backwards-compatibility to Perl 5.00503 and a critical
+bug fix to the 0.4.2 version of Net::Peep::Conf.
+
 Revision 1.6  2001/08/06 04:20:35  starky
 Fixed bug #447844.
 
